#!/bin/bash
# Display Clock on Frame Buffer

if [ $(basename $0) = $(basename ${BASH_SOURCE}) ]; then # 単独で実行
    echo "[Error] Cannot execute this script alone."
    exit 1
fi

if [ ${fb} -eq 3 ]; then # macOS
    echo "FBCMD=qlmanage"
    if [ ! -x /usr/local/bin/convert ] ; then
        echo "[Error] you need imagemagick"
        fb=0
        return $(false)
    fi
    if [ ! -x /usr/bin/qlmanage ] ; then
        echo "[Error] you need qlmanage"
        fb=0
        return $(false)
    fi

    if [ "x" = "x${DISPLAY}" ] ; then # DISPLAYがセットされていないなら終了
        echo "[Error] qamanage works on X."
        fb=0
        return $(false)
    fi

    if [ ! -x /usr/local/bin/gdate ] ; then
        echo "[Error] you need gdate"
        fb=0
        return $(false)
    fi

    # macOSでは、Font Pathは、Fullで指定
    efont="/Library/Fonts/Courier New Bold.ttf"
    jfont="${HOME}/Library/Fonts/migmix-1m-bold.ttf"
    dmode=$(system_profiler SPDisplaysDataType | awk '/Resolution:/{ print $2"x"$4}')
else
    if [ ! -x /bin/fbset ] ; then
        echo "[Error] you need: sudo apt install fbset"
        fb=0
        return $(false)
    fi
    if [ ! -x /usr/bin/convert ] ; then
        echo "[Error] you need imagemagick"
        fb=0
        return $(false)
    fi

    if [ ${fb} -eq 1 ]; then # fbi
        echo "FBCMD=fbi"
        if [ ! -x /usr/bin/fbi ] ; then
            echo "[Error] you need: sudo apt install fbi"
            fb=0
            return $(false)
        fi

        fbdev=/dev/fb0
        if [ ! -c ${fbdev} ] ; then # デバイスがないなら終了
            echo "[Error] You don't have the device ${fbdev}."
            fb=0
            return $(false)
        fi

        tty | fgrep '/dev/tty' > /dev/null 2>&1
        if [ $? -gt 0 ] ; then
            echo "[Error] Must execute on /dev/tty."
            fb=0
            return $(false)
        fi
    elif [ ${fb} -eq 2]; then # fim
        echo "FBCMD=fim"
        if [ ! -x /usr/bin/fim ] ; then
            echo "[Error] you need: sudo apt install fim"
            fb=0
            return $(false)
        fi

        if [ "x" = "x${DISPLAY}" ] ; then # DISPLAYがセットされていないなら終了
            echo "[Error] Fim works on X."
            fb=0
            return $(false)
        fi
    fi
    efont="Courier-Bold"
    jfont="MigMix-1M-Bold"
    dmode=$(fbset -fb /dev/fb0 | awk '/mode/ {gsub(/"/,"",$2);print $2}')
fi
dwidth=$(cut -f 1 -d x <<<${dmode})
dheight=$(cut -f 2 -d x <<<${dmode})

if [ ! -x /sbin/ifconfig ] ; then
    echo "[Warning] No ifconfig"
fi

if [ ${pm} -eq 1 ] ; then # Pwoer Save Management
    echo "[Info] Display and Wifi Power Management is ON." ;
    setterm -powersave off -blank 0 # Consoleなら省電力モードにしない
    gw=$(ip route | awk '/^default/ { print $3 }'a)
fi

if [ ${debug} -gt 0 ] ; then
    logfile="/tmp/fbclock.log"
else
    logfile="/dev/null"
fi

cat /dev/null > ${logfile}
echo > ${logfile}

png=/tmp/fbclock
pictdir=$(dirname $0)/pict
[ ! -e ${basepict} ] && unset basepict

# ImageMagickのidentifyコマンドで、対象画像を1つサンプリングして、サイズを算出するのがよいが...
pictres="2560x1600" # 画像サイズ Full=2560x1600ぐらい
if [ $dheight -le 480 ]; then # 画面サイズ(3.5inchとかの場合)から、画像をむりやり調整
    pictres="768x480"  # 画像サイズ Small=768x480ぐらい
fi
case ${pictres} in
    "2560x1600" )
      PSZC=72  # w/ IP,カレンダー,ニュース:フォントサイズ
      PSZD=160  # w/ 日時:フォントサイズ
      PSZW=63   # w/ 天気:フォントサイズ
      HOFFS=30  # w/o 日時:縦offset
      WOFFS=20  # w/o 天気:縦横offset
      let FSZC=dwidth/28   # w/o IP,カレンダー,ニュース:フォントサイズ
      let FSZD=dwidth/11   # w/o 日時:フォントサイズ
      let FSZW=dwidth/34   # w/o 天気:フォントサイズ  
      ;;
    "768x480"  )
      PSZC=25   # w/ IP,カレンダー,ニュース:フォントサイズ
      PSZD=40   # w/ 日時:フォントサイズ
      PSZW=23   # w/ 天気:フォントサイズ
      HOFFS=15  # w/o 日時:縦offset
      WOFFS=10  # w/o 天気:縦横offset
      let FSZC=dwidth/28   # w/o IP,カレンダー,ニュース:フォントサイズ
      let FSZD=dwidth/11   # w/o 日時:フォントサイズ
      let FSZW=dwidth/34   # w/o 天気:フォントサイズ  
      ;;
    *)           # default
      PSZC=72  # w/ IP,カレンダー,ニュース:フォントサイズ
      PSZD=160  # w/ 日時:フォントサイズ
      PSZW=63   # w/ 天気:フォントサイズ
      HOFFS=30  # w/o 日時:縦offset
      WOFFS=20  # w/o 天気:縦横offset
      let FSZC=dwidth/28   # w/o IP,カレンダー,ニュース:フォントサイズ
      let FSZD=dwidth/11   # w/o 日時:フォントサイズ
      let FSZW=dwidth/34   # w/o 天気:フォントサイズ  
      ;;
esac

myip=$(ifconfig ${nic} | awk '/inet /{print $2}')   # 使っている環境に合わせて wlan0 を書き換える

echo "dmode=$dmode"
echo "my ip address=$myip"

# ダブルバッファにして、display_clockからrendarをバックグラウンドで呼び出すようにする
declare -a dtime=(-1 -1)    # 画像バッファの状態
sr=58   # 描画を開始する秒
late=0  # 描画が遅延した回数
duration=59  # fb表示時間: fbi,fimの処理に時間がかかることを考慮
logflush=0 # Logファイルをフラッシュするためのカウンタ
wtext="" # ニュースもしくは、外部プログラムからのテキスト

set_trap () {
    if [ ${debug} -gt 0 ]; then
        if [ ${fb} -lt 3 ]; then
            trap 'dd if=/dev/zero of=${fbdev} ; setterm --default; reset ; exit' SIGINT SIGQUIT
        else
            trap 'dd if=/dev/zero of=${fbdev} ;  reset ; exit' SIGINT SIGQUIT
        fi
    else
        if [ ${fb} -lt 3 ]; then
            trap 'dd if=/dev/zero of=${fbdev} ; reset; setterm --default; rm ${png}?.png; exit' SIGINT SIGQUIT
        else
            trap 'dd if=/dev/zero of=${fbdev} ; reset; rm ${png}?.png; exit' SIGINT SIGQUIT
        fi
    fi
}

getwtext() {
    if [ ${nf} -gt 0 ]; then
        wtext=$(getnews)
        updatecnews
    elif [ -e ${rfile} ]; then
        wtext=$(<${rfile})
    else
        wtext=""
    fi
    echo "wtext=${cnews}:${wtext}:" >> ${logfile}
}

tic() {
    local pid r

    let logflush++
    if [ ${logflush} -gt 10080 ]; then # だいたい7日でログをフラッシュ
        cat /dev/null > ${logfile}
        logflush=0
    fi

    echo  >> ${logfile}
    echo $(${DATECMD} +"%_S") " > dtime=${dtime[@]}" >> ${logfile}

    [ ${dtime[0]} -eq -1 -o ${dtime[1]} -eq -1 ] && getwtext
    prepare_clock &
    pid=$!

    display_clock
    echo "=dtime=${dtime[@]}" >> ${logfile}

    if [ ${pm} -eq 1 ] ; then
        ping -c 1 $gw > /dev/null 2>&1 || echo "Network is DOWN" | tee -a ${logfile}
    fi

    if [ $(${DATECMD} +"%H") -eq ${nexthour} ]; then
        nexthour=$(( ( ($(${DATECMD} +"%_H") + 3) / 3 * 3) % 24 ))
        [ ${wf} -eq 1 ] && weather=$(getweather)
        [ ${nf} -gt 0 ] && feed
        echo "update weather&news : next update on ${nexthour} o'clock" >> ${logfile}
    fi

    wait ${pid}
    r=$?
    echo "    r=${r}" >> ${logfile}
    [ $r -ne 255 ] && dtime[$((r/60))]=$((r%60))
    echo $(${DATECMD} +"%_S") " < dtime=${dtime[@]}" >> ${logfile}
}


display_clock() {
    local buff  # バッファ読み込みID
    local dur
    local m=$(${DATECMD} +"%_M")

    echo "=display_clock @$(LC_TIME=C ${DATECMD})" >> ${logfile}

    for (( buff=0; buff<2; buff++ )) {
        echo "  buff=${buff}:Buff=${dtime[${buff}]} ? min:${m}" >> ${logfile}
        if [ ${dtime[${buff}]} -eq ${m} ] ; then    # 分がマッチ
            echo "    late=${late}" >> ${logfile}
            let late++
            if [ ${late} -gt 5 ]; then  # 遅延が頻出する場合、表示開始を早める
                let sr--
                [ ${sr} -lt 50 ] && sr=50
                late=0
            fi
            [ ${dtime[${buff}]} -eq 0 -a ${speak} -gt 0 ] && its
            break
        fi
        echo "    m=${m}, sr=${sr}" >> ${logfile}
        if [ $(${DATECMD} +"%_S") -ge ${sr} -a ${dtime[${buff}]} -eq $(((m+1)%60)) ] ; then    # 次分描画開始
            echo "    on time" >> ${logfile}
            [ ${dtime[${buff}]} -eq 0 -a ${speak} -gt 0 ] && its 1
            break
        fi
        # 著しく乖離しているときは、バッファを開放
        [ ${dtime[${buff}]} -ne $(((m+1)%60)) -a ${dtime[${buff}]} -ne $(((m+2)%60)) ] && let dtime[${buff}]=-1
    }
    if [ ${buff} -gt 1 ]; then # 描画対象の画像がなかった場合
        clear
        LC_TIME=C cal
        LC_TIME=C ${DATECMD}
        sleep 1
        return
    fi
    echo "    buff=${buff}" >> ${logfile}

    if [ -e "${png}${buff}.png" ]; then
        let dur=sr-$(${DATECMD} +"%_S")    # 表示が遅延するようなら、さらに減算する
        [ ${dur} -le 0 ] && dur=${duration}

        if [ ${fb} -eq 1 ]; then
            echo "fbi -d ${fbdev} -a --noverbose --noedit -t ${dur} -1 ${png}${buff}.png" >> ${logfile}
            fbi -d ${fbdev} -a --noverbose --noedit -t ${dur} -1 ${png}${buff}.png >> ${logfile} 2>&1
        elif [ ${fb} -eq 2 ]; then
            echo "fim -a -q -c 'sleep ${dur}; quit;' ${png}${buff}.png" >> ${logfile}
            #fim -a -q -o sdl=${dwidth}:${dheight} -c "sleep ${dur}; quit;" ${png}${buff}.png >> ${logfile} 2>&1
            fim -a -q -c "sleep ${dur}; quit;" ${png}${buff}.png >> ${logfile} 2>&1
        else
            echo "qlmanage -p ${png}${buff}.png" >> ${logfile}
            #( sleep ${dur} ; kill $(pgrep qlmanage) ) &
            #qlmanage -p ${png}${buff}.png >> ${logfile} 2>&1
            qlmanage -p ${png}${buff}.png >> ${logfile} 2>&1 &
            local qpid=$!
            sleep ${dur}; kill ${qpid}
        fi
    else
        echo "[Error] ${png}${buff}.png dose NOT exist." >> ${logfile}
    fi
    dtime[${buff}]=-1
}


prepare_clock() {
    local picts ctime cdate calend
    local m=$(${DATECMD} +"%_M")
    local s=$(${DATECMD} +"%_S")
    local buff  # バッファID
    local next=1    # next分先の絵を描画する
    local wd=${weather}

    echo "=prepare_clock ${m}:${s}">> ${logfile}

    for (( buff=0; buff<2; buff++ )) {
        if [ ${dtime[${buff}]} -eq -1 ]; then
            if [ ${dtime[$((buff^1))]} -eq $(((m+1)%60)) ]; then
                next=2
            fi
            break
        elif [ ${dtime[${buff}]} -eq $(((m+1)%60)) ]; then
            if [ ${dtime[$((buff^1))]} -eq -1 ]; then
                next=2
                let buff^=1
                break
            fi
        fi
    }
    [ ${buff} -gt 1 ] && return 255

    echo "preparing($m:$s) buff=${buff}, next=${next}">> ${logfile}

    if [ ${debug} -eq 2 ]; then # 強制背景あり
        picts=(${pictdir}/*.{jpg,png})
    elif [ ${debug} -eq 3 ]; then # 背景なし
        unset picts
    else
        [ $((RANDOM % 10)) -ne 0 ] && picts=(${pictdir}/*.{jpg,png})
    fi

    ctime="$(LC_TIME=C ${DATECMD} --date "${next} minutes" +"%H:%M%n%_d %A")"
    cdate="$(LC_TIME=C ${DATECMD} --date "${next} minutes" +"%F")"
    # Raspbianでは、cal -hができない ncal -Chも同じプログラム
    calend="$(LC_TIME=C cal -d ${cdate} | col -bx)"
    echo "rendering " $(LC_TIME=C ${DATECMD}) " for ${buff}:${dtime[${buff}]}">> ${logfile}

    # 以下 convert で、カレンダーを右寄せすると、7日無い週の表示位置がずれることに注意
    if [ ${#picts[@]} -gt 0 ]; then
        basepict=${picts[$((RANDOM % ${#picts[@]} ))]}
        echo "with picture:" ${basepict} >> ${logfile}
        convert \
            -font ${efont} \
            -pointsize ${PSZC} -fill "#c3e" -stroke "#315" -strokewidth 2 \
            -gravity NorthEast -annotate +20+20 "${myip}" \
            -pointsize ${PSZC} -fill "#ef6" -stroke "#321" -strokewidth 2 \
            -gravity SouthWest -annotate +20+20 "${calend}" \
            -pointsize ${PSZD} -fill "#4df" -stroke "#135" -strokewidth 2 \
            -gravity SouthEast -annotate +20+20 "${ctime}" \
            -font ${jfont} \
            -pointsize ${PSZC} -fill "#e8e" -stroke "#214" -strokewidth 2 \
            -gravity NorthEast -annotate +20+90 "${wtext}" \
            -pointsize ${PSZW} -fill "#fbf" -stroke "#213" -strokewidth 1 \
            -gravity NorthWest -annotate +20+60 "${wd}" \
            ${basepict} ${png}${buff}.png >> ${logfile} 2>&1
            # -kerning 2 \
    else
        echo "without picture" >> ${logfile}
        convert -size ${dmode} xc:transparent \
            -font ${efont} \
            -pointsize ${FSZC} -fill "#eee" -stroke "#222" -strokewidth 1 \
            -gravity NorthEast -annotate +20+20 "${myip}" \
            -pointsize ${FSZD} -fill blue -stroke "#02e" -strokewidth 1 \
            -gravity Center -draw "text 0,-${HOFFS} '${ctime}'" \
            -pointsize ${FSZC} -fill "#4df" -stroke "#135" -strokewidth 1 \
            -gravity SouthWest -annotate +10+10 "${calend}" \
            -font ${jfont} \
            -pointsize ${FSZC} -fill "#ddf" -stroke "#113" -strokewidth 1 \
            -gravity SouthEast -annotate +10+10 "${wtext}" \
            -pointsize ${FSZW} -fill "#d4f" -stroke "#315" -strokewidth 2 \
            -gravity NorthWest -annotate +${WOFFS}+${WOFFS} "${wd}" \
            ${png}${buff}.png >> ${logfile} 2>&1
    fi
    echo "rendered buff=${buff}:Buff=${dtime[${buff}]}" $(LC_TIME=C ${DATECMD}) " for ${ctime}">> ${logfile}
    # dtime[${buff}]=$((m+1)%60) バックグラウンドでは、親プロセスの変数は、変更できない
    return $(( ( (m+next) % 60 ) + (60 * buff) ))
}

echo "[Info] Frame buffer is ready."
true
