#!/bin/bash
# Display Clock on Frame Buffer

unset LC_ALL
export DATECMD=$(which date)
debug=0
fb=0    # Frame Buffer Mode (1でfbi使用, 2でfim使用:実験, 3でqlmanage)
pm=0    # Console Power Management (1でon)
wf=1    # Weather Forecasts (1でon)
nf=$(( $(tput cols) / 30 ))    # News Feed (nf>0でon) ; Yahoo Head Lineは高々30(chars/article)
speak=0 # Time Signal (0:off, 1:macos, 2:linux, 3:preparing)
nic="wlan0" # Network Interface (default:RPi)

rfile="/dev/null" #News Feedがoffの時表示するテキストデータ

usage_exit() {
    printf "%s [-fpwas] [-n #] [-d #] [i <interface>] [-r <file>]\n" $0
    echo "   -w : w/o Weather Forecasts"
    echo "   -a : CJK ambigous adjust for Weather Forecasts"
    echo "   -n # : display # of headlines"
    echo "   -r <file> : message file. (with out News Feed)"
    echo "   -s : Time Signal on"
    echo "   -f : Frame Buffer Mode"
    echo "   -F : Graphic Mode w/ fim (experimental)"
    echo "   -q : Graphic Mode for mac (experimental)"
    echo "   -p : Power Management on w/ Frame Buffer"
    echo "   -i <interface> : Network Interface name"
    echo "   -d # : Debug Level"
    exit
}

set_trap () {
    trap 'reset ; exit' SIGINT SIGQUIT
    printf "\033[?25l"  # カーソルを表示しない
}

display_clock() {
    local wd dt n nl
    local mon="" # debug目的で、表示月を変更するなら "-m 12" などと
    clear
    [ $debug -gt 0 ] && printf "[DEBUG] It's "
    [ $(${DATECMD} +"%M") -eq 0 -a ${speak} -gt 0 ] && its

    if [ ${nf} -gt 0 ]; then
        #echo -n "Headline News: "
        getnewsil ${nf}
    else
        [ -e ${rfile} ] && while read line ; do printf "%s " $line ; done < ${rfile}
    fi
    # calは、月の数が少ないときに、最後に空行を付け加えることがある
    ROWS=$(LC_TIME=C cal ${mon} | sed '/^[ \t]*$/d' | wc -l)
    echo
    for ((n=$( echo -e "${weather}" | wc -l) ; n < ROWS - 1; n++)) {
        nl="${nl} \n"
        #nl="${nl}_${n}\n"
    }
    IFS= dt=$(LC_TIME=C ${DATECMD} +"%_d %A%n%H:%M:%S" | display_right )
    wd="${weather}${nl}${dt}"
    let ROWS+=1
    paste <(LC_TIME=C cal ${mon} | sed '/^[ \t]*$/d') <( echo -e ${wd} | display_bottom )
    #echo ${ROWS}
}

tic() {
    printf "\033[%d;%dH:%s" $((ROWS)) $((COLS+22)) $(${DATECMD} +"%S")
    if [ $(${DATECMD} +"%S") -eq 0 ]; then
        if [ $(${DATECMD} +"%H") -eq ${nexthour} ]; then
            nexthour=$(( ( ($(${DATECMD} +"%H" | sed 's/^0//') + 3) / 3 * 3) % 24 ))
            [ ${wf} -eq 1 ] && weather=$(getweather)
            [ ${nf} -gt 0 ] && feed
        fi
        display_clock
    fi
    sleep 1
}

while getopts d:fFqpwasn:i:r: OPT
do
  case $OPT in
    d) debug=$(expr ${OPTARG} + 0) > /dev/null 2>&1
        if [ $? -gt 0 ]; then
            debug=0
        fi
        echo "option: Debug Level=${debug}"
        ;;
    f) fb=1; echo "option: Frame Buffer"; ;;
    F) fb=2; echo "option: Frame Buffer, w/ fim (experimental)"; ;;
    q) fb=3; echo "option: Graphic mode, for mac (experimental)"; ;;
    p) fb=1; pm=1; echo "option: Frame Buffer + Power Management"; ;;
    w) wf=0; echo "option: no Weather Forecasts"; ;;
    a) wf=1; ambwidth=" "; echo "option: CJK ambiguous adjust for Weather Forecasts"; ;;
    s) speak=3; echo "option: Time Signal"; ;;
    n) nf=${OPTARG}; echo "option: News Feed, get ${nf} headlines"; ;;
    i) nic=${OPTARG}; echo "watching NIC on ${nic}"; ;;
    r) rfile=${OPTARG}; nf=0; echo "reading messages from ${rfile}"; ;;
    \?) usage_exit ;;
  esac
done
shift $((OPTIND - 1))

#COLS="$(tput cols)"
# Sunday(6) Monday(6) Tuesday(7) Wednesday(9) Thursday(8) Friday(6) Saturday(7) + date(2) + SPC
COLS=12
wf_script=$(cd $(dirname $0); pwd)"/weather.bash"
if [ ${wf} -ne 0 -a -e ${wf_script} ]; then
    echo "Weather Forecasts"
    echo "reading ${wf_script}"
    source ${wf_script}
fi

nf_script=$(cd $(dirname $0); pwd)"/news.bash"
if [ ${nf} -gt 0 -a -e ${nf_script} ]; then
    echo "News Feed"
    echo "reading ${nf_script}"
    source ${nf_script}
fi

if [ ${wf} -eq 1 ]; then
    weather=$(getweather)
else
    weather=""
fi

if [ ${nf} -eq 0 ]; then
    echo "Assuming, messages are in ${rfile}."
else
    feed
fi
nexthour=$(( ( ($(${DATECMD} +"%H") + 3) / 3 * 3) % 24 ))

ts_script=$(cd $(dirname $0); pwd)"/timesignal.bash"
if [ ${speak} -gt 0 -a -e "${ts_script}" ]; then
    echo "Time Signal"
    echo "reading ${ts_script}"
    source ${ts_script}
fi

[ ${debug} -gt 0 ] && echo "[DEBUG] is Linux?"  # RPiもこれにあてはまる
uname | fgrep -i "linux" > /dev/null 2>&1
linux=$?
if [ ${debug} -gt 0 ]; then
    echo "[DEBUG] linux=${linux}"
fi
fb_script=$(cd $(dirname $0); pwd)"/fbclock.bash"
layout_script=$(cd $(dirname $0); pwd)"/layout.bash"
#if [ ${linux} -eq 0 -a -e ${fb_script} -a ${fb} -gt 0 ]; then
if [ -e ${fb_script} -a ${fb} -gt 0 ]; then
    echo "Get ready to draw pictures"
    echo "reading ${fb_script}"
    source ${fb_script}
else
    false
fi
if [ $? -gt 0 ]; then
    echo "Frame Buffer functions are disabled."
    fb=0
    if [ ! -e ${layout_script} ]; then
        echo "[Error] no layout script ${layout_script}."
        exit 1
    fi
    source ${layout_script}
fi

echo "----"

set_trap
# エラーの目視確認などのため、サービスを開始するまで、8sec待つ
for ((i=0; i<8; i++)) {
    echo -n ". "
    sleep 1
}

if [ -e /usr/local/bin/gdate ]; then # macではgdate(brew install coreutils)があれば
    export DATECMD=/usr/local/bin/gdate         # dateにgdateを用いるように
fi
# 0.0秒からはじめたい
sleep 0$(echo "1.000000000 - $(${DATECMD} +".%_N")" | bc)

display_clock

while :
do
    tic
done

