#!/bin/bash

if [ $(basename $0) = $(basename ${BASH_SOURCE}) ]; then # 単独で実行
    alone=1 
else
    alone=0
fi

if [ -x /usr/bin/say ] ; then
    echo "[Info] \"say\" is available. Yes, macOS."
    # available voice list : say -v ? | egrep 'en_'
    # Alex     ,en_US
    # Daniel   ,en_GB
    # Fred     ,en_US
    # Karen    ,en_AU
    # Moira    ,en_IE
    # Samantha ,en_US
    # Tessa    ,en_ZA
    # Veena    ,en_IN
    # Victoria ,en_US
    voice="Samantha"
    speak=1
    SPCMD="/usr/bin/say"
elif [ -x /usr/bin/espeak ] ; then
    echo "[Info] \"espeak\" is available. Yes, Linux."
    fgrep "Raspbian" /etc/issue > /dev/null 2>&1
    if [ $? -eq 0 ]; then # Raspbian
        RPI=1
        if [ ! -x /usr/bin/jackd ]; then
            echo "[Warning] No \"jackd\" even on Rpi, no \"Speak\" functions!"
            speak=0
            [ ${alone} -eq 0 ] && return $(false)
            exit 1
        fi
        ps ax | fgrep "jackd" | fgrep -v "grep" > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            echo 'starting "jackd"'
            jackd -d alsa -r 48000 -n 16 > /dev/null 2>&1 &
            jackd_pid=$!
            sleep 1
        fi
    fi
    # available voice list : espeak --voices | awk '$2 ~ /^en/{print $4}
    #voice="default"
    voice="english-us"
    speak=2
    SPCMD="/usr/bin/espeak"
else
    echo "[Warning] No \"Speak\" functions!"
    speak=0
    [ ${alone} -eq 0 ] && return $(false)
    exit 1
fi

if [ ${debug:-0} -gt 0 ] ; then
    speaklog="/tmp/timesignal.log"
else
    speaklog="/dev/null"
    #speaklog="/dev/stdout"
fi
cat /dev/null > ${speaklog}
echo > ${speaklog}

now() {
    if [ "x$1" = "x" ]; then
        printf "It's $(${DATECMD} +"%_H") o'clock."
    elif [ "x$1" = "x0" ]; then
        printf "It's $(${DATECMD} +"%_H:%_M")."
    elif [ ${speak} -gt 1 ]; then # might be linux only function
        printf "It's $(${DATECMD} --date "1 minutes" +"%_H") o'clock."
    fi
}

its() {
    now $1 | ${SPCMD} -v ${voice} >> ${speaklog} 2>&1 &
}

echo "I'll tell you the time." | ${SPCMD} -v ${voice} >> ${speaklog} 2>&1 
if [ ${alone} -eq 1 ]; then # 単独で実行
    unset alone
    if [ -e /usr/local/bin/gdate ]; then # macではgdate(brew install coreutils)があれば
        export DATECMD=/usr/local/bin/gdate         # dateにgdateを用いるように
    else
        export DATECMD=$(which date)
    fi
    now 0 ; echo
    its 0
    [ ${jackd_pid:-0} -ne 0 ] && kill ${jackd_pid}
else
    unset alone
    true
fi

