#!/bin/bash

if [ $(basename $0) = $(basename ${BASH_SOURCE}) ]; then # 単独で実行
    alone=1 
else
    alone=0
fi

if [ ! -x /usr/bin/jq -a ! -x /usr/local/bin/jq ] ; then
    echo "[Warning] No \"jq\"!  Weather Forecasts function is disabled."
    wf=0
    [ ${alone} -eq 0 ] && return $(false)
    exit 1
fi
jq --version 2>&1 | fgrep "1.3" > /dev/null 2>&1    # version 1.3はstderrにversionが出力
csvmode=$?  # jqのバージョンが1.3だったらcsvmode=0 -- on

if [ ! -x /usr/bin/curl ] ; then
    echo "[Warning] No \"curl\"! Weather Forecasts function is disabled."
    wf=0
    [ ${alone} -eq 0 ] && return $(false)
    exit 1
fi

# LOCALEが日本語で無い時はwf=0にすべきか、ここでLOCALEを設定する
export LANG=ja_JP.UTF-8 

numchar() {
    echo $* | grep -o '.' | wc -l | sed 's/^ *//'
}

nspace() {
    local i
    for (( i=0; i<$1; i++)) {
        printf "　"
    }
}

if [ ${csvmode} -gt 0 ]; then
getweather() {
    local line d w min max IFS=
    echo -e "　天気 (@$(${DATECMD} +"%H:%M"))         最低  最高"
    #cat ldweather_20180424.json |
    curl -s http://weather.livedoor.com/forecast/webservice/json/v1?city=130010 |
    jq -r '.forecasts[] | [.dateLabel, .telop, .temperature.min.celsius, .temperature.max.celsius] | @tsv' |
    while read line ; do
        d=$(cut -f 1 <<<${line})   # 今日, 明日, 明後日
        w=$(cut -f 2 <<<${line})   # 天気
        min=$(cut -f 3 <<<${line}) # 最低気温
        max=$(cut -f 4 <<<${line}) # 最高気温
        #printf "%s|%s|%s|%s|\n" ${d} ${w} ${min} ${max}

        printf "%s" ${d} ; printf "%*s" $(( 8 - $(numchar ${d})*2 ))
        printf "%s" ${w} ; printf "%*s" $(( 14 - $(numchar ${w})*2 ))
        #printf "|"

        if [ "x${min}" != "x" ]; then
            printf "  %2d℃%s" ${min} ${ambwidth}
        else
            printf "      "
        fi

        if [ "x${max}" != "x" ]; then
            printf "  %2d℃" ${max}
        else
            printf "      "
        fi
        #printf "|"
        printf "\n"
    done
}
else
getweather() {
    local line d w min max IFS=
    echo -e "　天気 (@$(${DATECMD} +"%H:%M"))         最低  最高"
    #cat ldweather_20180424.json |
    curl -s http://weather.livedoor.com/forecast/webservice/json/v1?city=130010 |
    jq -r '.forecasts[] | [.dateLabel, .telop, .temperature.min.celsius, .temperature.max.celsius] | @csv' |
    while read line ; do
        d=$(cut --delimiter="," -f 1 <<<${line} | sed 's/"//g')   # 今日, 明日, 明後日
        w=$(cut --delimiter="," -f 2 <<<${line} | sed 's/"//g')   # 天気
        min=$(cut --delimiter="," -f 3 <<<${line} | sed 's/"//g') # 最低気温
        max=$(cut --delimiter="," -f 4 <<<${line} | sed 's/"//g') # 最高気温
        #printf "%s|%s|%s|%s|\n" ${d} ${w} ${min} ${max}

        printf "%s" ${d} ; printf "%*s" $(( 8 - $(numchar ${d})*2 ))
        printf "%s" ${w} ; printf "%*s" $(( 14 - $(numchar ${w})*2 ))
        #printf "|"

        if [ "x${min}" != "x" ]; then
            printf "  %2d℃%s" ${min} ${ambwidth}
        else
            printf "      "
        fi

        if [ "x${max}" != "x" ]; then
            printf "  %2d℃" ${max}
        else
            printf "      "
        fi
        #printf "|"
        printf "\n"
    done
}
fi


if [ ${alone} -eq 1 ]; then # 単独で実行
    unset alone
    echo "天気取得中 ... csvmode=${csvmode} (0=on)"
    getweather
else
    unset alone
    echo "[Info] getting weather information."
    wf=1
    true
fi

